import org.junit.Test;
import org.junit.After;
import java.lang.reflect.Field;
import org.junit.Assert;
import org.junit.Before;
import org.junit.rules.Timeout;

import org.junit.runner.JUnitCore;
import org.junit.runner.Result;
import org.junit.runner.notification.Failure;

import javax.swing.*;
import java.io.*;
import java.lang.reflect.*;
import java.util.ArrayList;
import java.util.concurrent.ThreadLocalRandom;
import java.lang.reflect.InvocationTargetException;
import java.util.UUID;


import static org.junit.Assert.*;

/**
 * A local test to ensure the supporting documents work properly
 *
 * <p>Purdue University -- CS18000 -- Spring 2021</p>
 *
 * @author Weihang Li
 * @version May 1st, 2021
 */

public class RunLocalTest {
    public static void main(String[] args) {
        Result result = JUnitCore.runClasses(TestCase.class);
        if (result.wasSuccessful()) {
            System.out.println("Excellent - Test ran successfully");
        } else {
            for (Failure failure : result.getFailures()) {
                System.out.println(failure.toString());
            }
        }
    }

    /**
     * A local test to ensure the supporting documents work properly
     *
     * <p>Purdue University -- CS18000 -- Spring 2021</p>
     *
     * @author Weihang Li
     * @version May 1st, 2021
     */

    public static class TestCase {
        private final PrintStream originalOutput = System.out;
        private final InputStream originalSysin = System.in;

        @SuppressWarnings("FieldCanBeLocal")
        private ByteArrayInputStream testIn;

        @SuppressWarnings("FieldCanBeLocal")
        private ByteArrayOutputStream testOut;

        @Before
        public void outputStart() {
            testOut = new ByteArrayOutputStream();
            System.setOut(new PrintStream(testOut));
        }

        @After
        public void restoreInputAndOutput() {
            System.setIn(originalSysin);
            System.setOut(originalOutput);
        }

        private String getOutput() {
            return testOut.toString();
        }

        @SuppressWarnings("SameParameterValue")
        private void receiveInput(String str) {
            testIn = new ByteArrayInputStream(str.getBytes());
            System.setIn(testIn);
        }




        @Test(timeout = 1000)
        public void ChatInfoClassDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = ChatInfo.class;

            // Perform tests

            modifiers = clazz.getModifiers();

            Assert.assertTrue("Ensure that `ChatInfo` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `ChatInfo` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void FindChatDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = FindChat.class;

            // Perform tests

            modifiers = clazz.getModifiers();


            Assert.assertTrue("Ensure that `FindChat` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `FindChat` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void MessageDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = Message.class;

            // Perform tests

            modifiers = clazz.getModifiers();



            Assert.assertTrue("Ensure that `Message` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `Message` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void ReadUserNameDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = ReadUserName.class;

            // Perform tests

            modifiers = clazz.getModifiers();

            Assert.assertTrue("Ensure that `ReadUserName` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `ReadUserName` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void UserDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = User.class;

            // Perform tests

            modifiers = clazz.getModifiers();

            Assert.assertTrue("Ensure that `User` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `User` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void WriteFileDeclarationTest() {
            Class<?> clazz;
            int modifiers;

            // Set the class being tested
            clazz = Message.class;

            // Perform tests

            modifiers = clazz.getModifiers();



            Assert.assertTrue("Ensure that `WriteFile` is `public`!", Modifier.isPublic(modifiers));

            Assert.assertFalse("Ensure that `WriteFile` is NOT `abstract`!", Modifier.isAbstract(modifiers));

        }

        @Test(timeout = 1000)
        public void testChatInfo() {

            String line1 = "1!Bob,Leo!2021-04-21 15:35:40.482!Bob,Leo";

            ChatInfo info = new ChatInfo(line1);
            ArrayList<String> users = new ArrayList<>();
            users.add("Bob");
            users.add("Leo");

            assertEquals("Ensure that you split index correctly ", info.getIndex(), 1);
            assertEquals("Ensure that you split users correctly", info.getChatUser(), users);
        }

        @Test(timeout = 1000)
        public void testFindChat() {

            String MyUserName = "Leo";

            FindChat find = new FindChat(MyUserName);

            assertEquals("Ensure that you Find the correct index", find.findIndex("team4"), 4);
            assertEquals("Ensure that you Find the correct index", find.findIndex("Anna,Leo"), 3);
        }

        @Test(timeout = 1000)
        public void testUser() {

            String UserName = "Leo";
            String Password = "12345";

            User user = new User (UserName, Password);

            assertEquals("Ensure that you return the correct value", user.getUsername(), "Leo");
            assertEquals("Ensure that you Find the correct index", user.getPassword(), "12345");
        }
    }
}
